@@ -18,6 +18,7 @@ sub import {
 	hook(type => "rcs", id => "rcs_recentchanges", call => \&rcs_recentchanges);
 	hook(type => "rcs", id => "rcs_diff", call => \&rcs_diff);
 	hook(type => "rcs", id => "rcs_getctime", call => \&rcs_getctime);
+	hook(type => "rcs", id => "rcs_getmtime", call => \&rcs_getmtime);
 }
 
 sub silentsystem (@) {
@@ -51,7 +52,7 @@ sub darcs_info ($$$) {
 	return $_;
 }
 
-sub file_in_vc($$) {
+sub file_in_vc ($$) {
 	my $repodir = shift;
 	my $file = shift;
 
@@ -62,14 +63,14 @@ sub file_in_vc($$) {
 	}
 	my $found=0;
 	while (<DARCS_MANIFEST>) {
-		$found = 1, last if /^(\.\/)?$file$/;
+		$found = 1 if /^(\.\/)?$file$/;
 	}
 	close(DARCS_MANIFEST) or error("'darcs query manifest' exited " . $?);
 
 	return $found;
 }
 
-sub darcs_rev($) {
+sub darcs_rev ($) {
 	my $file = shift; # Relative to the repodir.
 	my $repodir = $config{srcdir};
 
@@ -78,7 +79,7 @@ sub darcs_rev($) {
 	return defined $hash ? $hash : "";
 }
 
-sub checkconfig() {
+sub checkconfig () {
 	if (defined $config{darcs_wrapper} && length $config{darcs_wrapper}) {
 		push @{$config{wrappers}}, {
 			wrapper => $config{darcs_wrapper},
@@ -87,7 +88,7 @@ sub checkconfig() {
 	}
 }
 
-sub getsetup() {
+sub getsetup () {
 	return
 		plugin => {
 			safe => 0, # rcs plugin
@@ -238,7 +239,7 @@ sub rcs_commit ($$$;$$) {
 	}
 }
 
-sub rcs_commit_staged($$$) {
+sub rcs_commit_staged ($$$) {
 	my ($message, $user, $ipaddr) = @_;
 
 	my $author;
@@ -427,4 +428,8 @@ sub rcs_getctime ($) {
 	return $date;
 }
 
+sub rcs_getmtime ($) {
+	error "rcs_getmtime is not implemented for darcs\n"; # TODO
+}
+
 1
