@@ -23,6 +23,14 @@ sub getsetup () {
 sub parentlinks ($) {
 	my $page=shift;
 
+	if (! length $page) {
+		# dynamic page
+		return {
+			url => $config{url},
+			page => $config{wikiname},
+		};
+	}
+
 	my @ret;
 	my $path="";
 	my $title=$config{wikiname};
@@ -53,12 +61,11 @@ sub parentlinks ($) {
 
 sub pagetemplate (@) {
 	my %params=@_;
-        my $page=$params{page};
         my $template=$params{template};
 
 	if ($template->query(name => "parentlinks") ||
  	   $template->query(name => "has_parentlinks")) {
-		my @links=parentlinks($page);
+		my @links=parentlinks($params{page});
 		$template->param(parentlinks => \@links);
 		$template->param(has_parentlinks => (@links > 0));
 	}
